from Fusion.ui.menu import Menu
from Fusion.download.downloader import Downloader
from Fusion.Packages.packager import Packager
import json
class Fusion:
    def __init__(self):
        version = self.get_version()
        logo = f"""
       ______           _                
      / ____/_  _______(_)___  ____      
     / /_  / / / / ___/ / __ \/ __ \     
    / __/ / /_/ (__  ) / /_/ / / / /     
   /_/    \__,_/____/_/\____/_/ /_/      
        powered by f09l
        version {version}                               
        """                        
            
        main_menu = Menu([
            ["Downloader", lambda: Downloader(main_menu=main_menu)],
            ["Packages", lambda: Packager(main_menu=main_menu)],
            ["Exit", lambda:exit()]
        ],logo=logo)

        # Start the main menu
        main_menu.start()
    def get_version(self):
        try:
            with open("version.json") as f:
                    return json.load(f)["version"]
        except:
             return ""
Fusion()


from blessed import Terminal
import os
import time
import requests
from ui.menu import Menu

class Downloader:
    def __init__(self, main_menu):
          self.main_menu = main_menu
          self.menu = Menu([
            ["Download Repository", self.select_repository],
            ["Back", self.exit_action]
        ], main_menu=self.main_menu)
          self.menu.start()

    def select_repository(self):
        username = self.menu.get_input("Enter the GitHub username: ")
        api_url = f"https://api.github.com/users/{username}/repos"
        response = requests.get(api_url)
        if response.status_code == 200:
            repositories = response.json()
            options = [[repo["name"], lambda repo=repo: self.download_repository(repo["html_url"])] for repo in repositories]
            options.append(["Back", self.menu.start])
            repo_menu = Menu(options, parent_menu=self.menu)
            repo_menu.start()
        else:
            print(f"Failed to fetch repositories for '{username}'.")
            self.menu.start()

    def download_repository(self, url):
        username, repo_name = url.split("/")[-2:]
        api_url = f"https://api.github.com/repos/{username}/{repo_name}/contents"
        print(f"Downloading '{repo_name}'")
        response = requests.get(api_url)
        if response.status_code == 200:
            contents = response.json()
            repo_directory = os.path.join("Packages", repo_name)
            os.makedirs(repo_directory, exist_ok=True)

            for item in contents:
                if item["type"] == "file":
                    file_path = item["path"]
                    file_url = item["download_url"]
                    save_path = os.path.join(repo_directory, file_path)

                    os.makedirs(os.path.dirname(save_path), exist_ok=True)

                    self.download_file(file_url, save_path)

            print(f"Repository '{repo_name}' downloaded.")
        else:
            print(f"Failed to fetch repository contents for '{repo_name}'.")

        time.sleep(5)

    def download_file(self, file_url, save_path):
        response = requests.get(file_url)
        response.raise_for_status()

        with open(save_path, "wb") as file:
            file.write(response.content)

    def exit_action(self):
        self.main_menu.start()




from blessed import Terminal
from colorama import Fore, Style,Back
import time
class Menu:
    def __init__(self, options,logo="",parent_menu=None,main_menu=None):
        self.term = Terminal()
        self.options = options
        self.selection = 0
        self.logo = logo
        self.parent_menu = parent_menu
        self.main_menu = main_menu
    def create_gradient_logo(self):
        gradient_logo = ""
        gradient_step = 255 // (len(self.logo.splitlines()) - 1)
        for i, line in enumerate(self.logo.splitlines()):
            color = f"\033[38;2;255;{255 - (gradient_step * i)};0m"
            gradient_logo += f"{color}{line}{Style.RESET_ALL}\n"
        return gradient_logo

    def display_screen(self):
        print(self.term.clear())
        gradient_logo = self.create_gradient_logo()
        print(gradient_logo)

        for idx, item in enumerate(self.options):
            if idx == self.selection:
                print(self.term.reverse(f'{self.term.bold_green}{item[0]:<5}'))
            else:
                print(f'{self.term.normal}{item[0]:<15}')
    def run_selection(self):
        action = self.options[self.selection][2]
        if action:
            self.display_screen()

            with self.term.fullscreen(), self.term.cbreak():
                action()
            



    def handle_input(self, key):
        if key.is_sequence:
            if key.name == 'KEY_DOWN':
                self.selection = (self.selection + 1) % len(self.options)
            elif key.name == 'KEY_UP':
                self.selection = (self.selection - 1) % len(self.options)
            elif key.name == 'KEY_ENTER':
                if self.selection == len(self.options) - 1:  # Exit option
                    self.handle_exit()
                else:
                    action = self.options[self.selection][1]
                    if action:
                        self.display_screen()
                        with self.term.fullscreen(), self.term.cbreak():
                            action()
                        self.display_screen()  # Redraw menu after action
        elif key:
            print(f"Got {key}.")
            self.display_screen()

    def handle_exit(self):
        if self.parent_menu:
            self.parent_menu.start()
        else:
            try:
                self.main_menu.start()
            except:
                exit()


    def get_input(self, prompt):
        with self.term.cbreak():
            print(prompt, end="", flush=True)
            input_text = ""
            while True:
                key = self.term.inkey()
                if key.is_sequence:
                    if key.name == "KEY_ENTER":
                        print()
                        break
                    elif key.name == "KEY_BACKSPACE":
                        if len(input_text) > 0:
                            input_text = input_text[:-1]
                            print("\b \b", end="", flush=True)
                    else:
                        input_text += key
                        print(key, end="", flush=True)
                elif key:
                    input_text += key
                    print(key, end="", flush=True)

            return input_text

    def start(self):
        with self.term.fullscreen(), self.term.cbreak():
            while True:
                self.display_screen()
                key = self.term.inkey()
                self.handle_input(key)
                


import os
import json
import time
from ui.menu import Menu

class Packager:
    def __init__(self, main_menu):
        self.main_menu = main_menu
        self.menu = Menu([], parent_menu=self.main_menu)
        self.run()

    def get_folders(self):
        current_path = os.path.join(os.getcwd(), "Packages")
        if not os.path.exists(current_path):
            return []
        folders = [name for name in os.listdir(current_path) if os.path.isdir(os.path.join(current_path, name))]
        return folders

    def select_folder(self, folder):
        folder_path = os.path.join(os.getcwd(), "Packages", folder)
        json_file_path = os.path.join(folder_path, "Fusion.json")

        if os.path.exists(json_file_path):
            with open(json_file_path, "r") as file:
                try:
                    data = json.load(file)
                    main_file = data.get("main")
                    if main_file:
                        main_file_path = os.path.join(folder_path, main_file)
                        if os.path.exists(main_file_path):
                            # Run the main Python file
                            os.system(f"python {main_file_path}")
                        else:
                            print("Main Python file not found.")
                    else:
                        print("No 'main' key found in Fusion.json.")
                except json.JSONDecodeError:
                    print("Error parsing Fusion.json.")
        else:
            print("Fusion.json file not found.\nmake sure you have Fusion.json file in your folder with the main file path ")

        time.sleep(2)
        self.menu.start()

    def run(self):
        folders = self.get_folders()
        options = [[folder, lambda folder=folder: self.select_folder(folder)] for folder in folders]
        options.append(["Back", self.menu.start])
        self.menu.options = options
        self.menu.start()




from setuptools import setup, find_packages

setup(
    name='ufusion',
    version='1.0.8',
    description='',
    author='f09l',
    author_email='fostn.contact@gmail.com',
    packages=find_packages(),
    install_requires=['requests','colorama','blessed'],
    entry_points={
        'console_scripts': [
            'fusion=Fusion.Fusion:main',
        ],
    },
)
